<?php
/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Slavenja <slavenja@ya.ru>
 *  @copyright 20019-2020 intopsite.ru
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class csvexpert extends Module
{
    public static $csv_dir = '../modules/csvexpert/csv';
    protected static $config_form = array(
        'TREE_CAT_CSVEXPERT',
    );

    public function __construct()
    {
        $this->name = 'csvexpert';
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->author = 'intopsite.ru';
        $this->need_instance = 1;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('CSVexpert');
        $this->description = $this->l('Export all Prestashop data to CSV file');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('TREE_CAT_CSVEXPERT', '2');

        return parent::install();
    }

    public function uninstall()
    {
        foreach (csvexpert::$config_form as $field) {
            Configuration::deleteByName($field);
        }

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool) Tools::isSubmit('submitcsvexpertModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $this->context->smarty->assign('result', $this->_listFiles());

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');
        $id_lang = $this->context->language->id;


        return $this->renderForm() . $output;
    }

    protected function _listFiles()
    {
        $url = Tools::getShopDomainSsl(true);
        $csv_dir = csvexpert::$csv_dir;

        if (is_dir($csv_dir)) {
            if ($dh = opendir($csv_dir)) {
                while (($file = readdir($dh)) !== false) {
                    if ($file != '.' && $file != '..' && substr($file, strlen($file) - 3) == 'csv') {
                        $kb = number_format(filesize($csv_dir . '/' . $file) / 1024, 2);
                        $result .= '<div class="panel"><fieldset><div class="col-lg-3"></div> <div class="col-lg-9"><div class="col-lg-6"><a class="btn btn-primary btn-lg btn-block" href=' . $url . '/' . $csv_dir . '/' . $file . '><i class="icon-chevron-sign-down"></i>  ' . $file . '</a> <b>' . date('d.m.Y H:i', filemtime($csv_dir . '/' . $file)) . ' | ' . $kb . ' kB </b></div></div><fieldset></div>';
                    }
                }
                closedir($dh);
            }
        }
        return $result;
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitcsvexpertModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
        . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 6,
                        'type' => 'categories',
                        'label' => $this->l('Select Categories'),
                        'name' => 'TREE_CAT_CSVEXPERT',
                        'tree' => array(
                            'selected_categories' => explode(',', Configuration::get('TREE_CAT_CSVEXPERT')),
                            'disabled_categories' => null,
                            'use_search' => true,
                            'use_checkbox' => true,
                            'id' => 'id_category_tree',
                        ),
                    ),
                ),
                'buttons' => array(
                    'save-and-stay' => array(
                        'title' => $this->l('x'),
                        'name' => 'btnDelete',
                        'type' => 'submit',
                        'class' => 'btn btn-danger pull-right',
                    ), array(
                        'title' => $this->l('Manufacturers'),
                        'name' => 'btnManufacturers',
                        'type' => 'submit',
                        'class' => 'btn btn-success  pull-right',
                    ), array(
                        'title' => $this->l('Attributes'),
                        'name' => 'btnAttributes',
                        'type' => 'submit',
                        'class' => 'btn btn-success  pull-right',
                    ),
                    array(
                        'title' => $this->l('Categories'),
                        'name' => 'btnCategories',
                        'type' => 'submit',
                        'class' => 'btn btn-success  pull-right',
                    ),
                    array(
                        'title' => $this->l('Min Products'),
                        'name' => 'btnMinProd',
                        'type' => 'submit',
                        'class' => 'btn btn-success  pull-right',
                    ),
                    array(
                        'title' => $this->l('Export Products'),
                        'name' => 'btnProducts',
                        'type' => 'submit',
                        'class' => 'btn btn-success  pull-right',
                    ),
                ),
                'submit' => array(
                    'title' => $this->l(''),
                    'class' => 'btn btn-info btn-bloc pull-right',
                    'icon' => 'icon icon-save',
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'TREE_CAT_CSVEXPERT' => Configuration::get('TREE_CAT_CSVEXPERT'),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {

        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {

            // Configuration::updateValue($key, Tools::getValue($key));

            if ($TREE_CAT_CSVEXPERT = Tools::getValue('TREE_CAT_CSVEXPERT')) {
                Configuration::updateValue('TREE_CAT_CSVEXPERT', implode(',', $TREE_CAT_CSVEXPERT));
            } else {
                Configuration::updateValue('TREE_CAT_CSVEXPERT', '');
            }
        }

        if (Tools::isSubmit('btnCategories')) {
            $this->csvCategories();
        } elseif (Tools::isSubmit('btnProducts')) {
            $this->csvProducts();
        } elseif (Tools::isSubmit('btnMinProd')) {
            $this->csvMinprod();
        } elseif (Tools::isSubmit('btnAttributes')) {
            $this->csvAttributes();
        } elseif (Tools::isSubmit('btnManufacturers')) {
            $this->csvManufacturers();
        } elseif (Tools::isSubmit('btnDelete')) {
            $this->csvDelete();
        }

    }

    /**
     * Clean category
     */
    protected function cleanCat($string)
    {
        $pr = array(" ,");
        $replace = ',';
        $string = str_replace($pr, $replace, $string);
        $string = trim(mb_ereg_replace('\\s+', ' ', $string));
        return $string;
    }

    /**
     * Clean Description <i><em><strong><b>
     */
    protected function cleanDesc($string)
    {
        $pr = array("\r\n", "\r", "\n", "\t");
        $replace = '<br />';
        $string = str_replace($pr, $replace, $string);
        $string = str_replace('<br /><br />', '<br />', $string);
        $string = str_replace('<p></p>', '', $string);
        $allowed_tags = '<img><br><hr><p><h2><h3><h4><h5><h6><sup><sub><ul><ol><li><table><tr><td>';
        $string = strip_tags($string, $allowed_tags);
        $string = str_replace('&nbsp;', ' ', $string);
        $string = trim(mb_ereg_replace('\\s+', ' ', $string));
        $string = htmlspecialchars_decode($string);
        $string = preg_replace('/"([^"]*)"/', "«$1»", $string);
        $string = preg_replace("/'([^']*)'/", "«$1»", $string);
        return $string;
    }

    /**
     * Clean Short description
     */
    protected function cleanShortDesc($string)
    {
        // Меняем на пробел
        $replace = array("<br/>", "<br />", "<br>", "<p>", "</p>", "&nbsp;", "\r\n", "\r", "\n", "\t");
        $string = str_replace($replace, ' ', $string);
        // Удаляем теги
        $string = strip_tags($string);
        // Меняем несколько пробелов на 1
        $string = trim(mb_ereg_replace('\\s+', ' ', $string));
        // Преобразует HTML-сущности в символы
        $string = htmlspecialchars_decode($string);
        // замена кавычек "" '' на «»
        $string = preg_replace('/"([^"]*)"/', "«$1»", $string);
        $string = preg_replace("/'([^']*)'/", "«$1»", $string);
        return $string;
    }

    /**
     * Clean Keywords
     */
    protected function cleanKeywords($string)
    {
        $string = mb_strtolower($string, 'utf-8');
        $pr = array("\r\n", "\r", "\n", "\t");
        $replace = ' ';
        $string = str_replace($pr, $replace, $string);
        $string = strip_tags($string);
        $string = str_replace('&nbsp;', ' ', $string);
        $string = str_replace(',', ', ', $string);
        $string = trim(mb_ereg_replace('\\s+', ' ', $string));
        return $string;
    }

    /**
     * Get categories and subcategories id
     */
    protected function getCatSubcatId($cat_id)
    {
        $sql = 'SELECT c.`id_category`';
        $sql .= 'FROM `' . _DB_PREFIX_ . 'category` c ';
        $sql .= 'RIGHT JOIN `' . _DB_PREFIX_ . 'category` c2 ON c2.`id_category` IN (' . $cat_id . ') AND c.`nleft` >= c2.`nleft` ';
        $sql .= 'AND c.`nright` <= c2.`nright` ';
        $sql .= 'GROUP BY c.`id_category` ORDER BY c.`nleft` ASC ';
        $cat_id = implode(',', array_map(function ($value) {
            return $value['id_category'];
        }, Db::getInstance()->ExecuteS($sql)));
        $cat_ids = ltrim($cat_id, ',');
        return $cat_ids;
    }

    /**
     * Get Categories
     */
    protected function csvCategories()
    {
        require_once 'csvwrite.php';
        $csv_dir = csvexpert::$csv_dir . '/';
        $csv = new CSV($csv_dir, 'categoryexport.csv');
        $title = array('Category ID', 'Active (0/1)', 'Name *', 'Parent category', 'Root category (0/1)', 'Description', 'Meta title', 'Meta keywords', 'Meta description', 'URL rewritten', 'Image URL');
        $csv->setCSV(array(0 => $title));

        $cat_id = Configuration::get('TREE_CAT_CSVEXPERT');
        $cat_ids = $this->getCatSubcatId($cat_id);
        if ($cat_id) {
            $url = Tools::getShopDomainSsl(true);
            $id_lang = $this->context->language->id;
            $sql = 'SELECT c.`id_category`, c.`active`, cl.`name`, cl2.`name` AS `parent_name` , c.`is_root_category`, cl.`description`, cl.`meta_title`, cl.`meta_keywords`, cl.`meta_description`, cl.`link_rewrite` ';
            $sql .= 'FROM `' . _DB_PREFIX_ . 'category` c ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl ON (c.`id_category` = cl.`id_category` AND cl.`id_lang` = ' . $id_lang . ') ';
            $sql .= 'INNER JOIN `' . _DB_PREFIX_ . 'category_lang` cl2 ON (c.`id_parent` = cl2.`id_category` AND cl.`id_lang` = ' . $id_lang . ') ';
            $sql .= 'WHERE c.`id_category` IN (' . $cat_ids . ') ';
            $sql .= 'GROUP BY c.`id_category` ORDER BY c.`is_root_category` DESC , `parent_name`, cl.`name`';
            $catscats = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS($sql);

            foreach ($catscats as $key => $value) {
                if ($catscats[$key]['is_root_category'] == '1') {
                    unset($catscats[$key]);
                }
                if ($catscats[$key]['name']) {
                    $catscats[$key]['name'] = trim(mb_ereg_replace('\\s+', ' ', $value['name']));
                }
                if ($catscats[$key]['parent_name']) {
                    $catscats[$key]['parent_name'] = trim(mb_ereg_replace('\\s+', ' ', $value['parent_name']));
                }
                if ($catscats[$key]['description']) {
                    $catscats[$key]['description'] = $this->cleanDesc($value['description']);
                }
                if ($catscats[$key]['title']) {
                    $catscats[$key]['title'] = $this->cleanShortDesc($value['title']);
                }
                if ($catscats[$key]['meta_keywords']) {
                    $catscats[$key]['meta_keywords'] = $this->cleanKeywords($value["meta_keywords"]);
                }
                if (file_exists(_PS_CAT_IMG_DIR_ . (int) $catscats[$key]['id_category'] . '.jpg')) {
                    $catscats[$key]['image'] = $url . '/img/c/' . $catscats[$key]['id_category'] . '.jpg';
                } elseif (file_exists(_PS_CAT_IMG_DIR_ . (int) $catscats[$key]['id_category'] . '-medium_default.jpg')) {
                    $catscats[$key]['image'] = $url . '/img/c/' . $catscats[$key]['id_category'] . '-medium_default.jpg';
                } elseif (file_exists(_PS_CAT_IMG_DIR_ . (int) $catscats[$key]['id_category'] . '_thumb.jpg')) {
                    $catscats[$key]['image'] = $url . '/img/c/' . $catscats[$key]['id_category'] . '_thumb.jpg';
                } else {
                    $catscats[$key]['image'] = '';
                }
            }

            $csv->setCSV($catscats);
        }
    }

    /**
     * Get Products
     */
    protected function csvProducts()
    {
        require_once 'csvwrite.php';
        $csv_dir = csvexpert::$csv_dir . '/';
        $csv = new CSV($csv_dir, 'productexport.csv');
        $title = array('Product ID', 'Active (0/1)', 'Name *', 'Categories (x,y,z...)', 'Price tax excluded or Price tax included', 'Tax rules ID', 'Wholesale price', 'On sale (0/1)', 'Discount amount', 'Discount percent', 'Discount from (yyyy-mm-dd)', 'Discount to (yyyy-mm-dd)', 'Reference #', 'Supplier reference #', 'Supplier', 'Manufacturer', 'EAN13', 'UPC', 'Ecotax', 'Width', 'Height', 'Depth', 'Weight', 'Quantity', 'Minimal quantity', 'Visibility', 'Additional shipping cost', 'Unity', 'Unit price', 'Short description', 'Description', 'Tags (x,y,z...)', 'Meta title', 'Meta keywords', 'Meta description', 'URL rewritten', 'Text when in stock', 'Text when backorder allowed', 'Available for order (0=No, 1=Yes)', 'Product available date', 'Product creation date', 'Show price (0=No, 1=Yes)', 'Image URLs (x,y,z...)', 'Delete existing images (0=No, 1=Yes)', 'Feature(Name:Value:Position)', 'Available online only (0=No, 1=Yes)', 'Condition', 'Customizable (0=No, 1=Yes)', 'Uploadable files (0=No, 1=Yes)', 'Text fields (0=No, 1=Yes)', 'Out of stock', 'ID / Name of shop', 'Advanced stock management', 'Depends On Stock', 'Warehouse');
        $csv->setCSV(array(0 => $title));

        $cat_id = Configuration::get('TREE_CAT_CSVEXPERT');
        $cat_ids = $this->getCatSubcatId($cat_id);
        if ($cat_id) {
            $url = Tools::getShopDomainSsl(true);
            $id_lang = (int) $this->context->language->id;
            $id_shop = (int) $this->context->shop->id;
            $delete_existing_images = '1';
            // $last_id = 0;
            // $memstart = memory_get_usage();
            // do {
            $sql = 'SELECT p.`id_product`, p.`active`, pl.`name`, ';
            $sql .= 'GROUP_CONCAT(DISTINCT(cl.`name`) ORDER BY p.`id_category_default` ASC SEPARATOR ",") AS `category`, ';
            $sql .= 'p.`price`, p.`id_tax_rules_group`, p.`wholesale_price`, p.`on_sale`, ';
            $sql .= 'IF(sp.`reduction_type`="amount", sp.`reduction`, "") AS `discount amount`, ';
            $sql .= 'IF(sp.`reduction_type`="percentage", sp.`reduction`, "") AS `discount percent`, ';
            $sql .= 'sp.`from`, sp.`to`, p.`reference`, ';
            $sql .= 'ps.`product_supplier_reference` AS `supplier reference`, ';
            $sql .= 'su.`name` AS `supplier`, ';
            $sql .= 'm.`name` AS `manufacturer`, ';
            $sql .= 'p.`ean13`, p.`upc`, p.`ecotax`, p.`width`, p.`height`, p.`depth`, p.`weight`, ';
            $sql .= 'psa.`quantity`, ';
            $sql .= 'p.`minimal_quantity`, p.`visibility`, p.`additional_shipping_cost`, p.`unity`, p.`unit_price_ratio`, ';
            $sql .= 'pl.`description_short`, pl.`description`, ';
            $sql .= 'IF(t.`name` IS NOT NULL, GROUP_CONCAT(DISTINCT(t.`name`) SEPARATOR ","), "") AS `tags`, ';
            $sql .= 'pl.`meta_title`, pl.`meta_keywords`, pl.`meta_description`, pl.`link_rewrite`, ';
            $sql .= 'pl.`link_rewrite`, pl.`available_now`, pl.`available_later`, ';
            $sql .= 'p.`available_for_order`, p.`available_date`, p.`date_add`, p.`show_price`, ';
            $sql .= 'IF(pi.`id_product` IS NOT NULL, GROUP_CONCAT(DISTINCT(CONCAT( "' . $url . '/img/p/",MID(pi.id_image,1,1),"/", IF (LENGTH(pi.id_image)>1, CONCAT(MID(pi.id_image,2,1),"/"),""),IF (LENGTH(pi.id_image)>2,CONCAT(MID(pi.id_image,3,1),"/"),""),IF (LENGTH(pi.id_image)>3,CONCAT(MID(pi.id_image,4,1),"/"),""),IF (LENGTH(pi.id_image)>4,CONCAT(MID(pi.id_image,5,1),"/"),""), pi.id_image, ".jpg" )) ORDER BY pi.`cover` DESC, pi.`position` ASC SEPARATOR ","), "") AS url_image, ';
            $sql .= 'IF(pi.`id_product` IS NOT NULL,' . $delete_existing_images . ', "") AS `delete existing img (0=no, 1=yes)`, ';
            $sql .= 'GROUP_CONCAT(DISTINCT(CONCAT((fl.`name`), ":", (REPLACE(fvl.`value`,","," &")), ":" , (f.position))) SEPARATOR ",") AS `feature(name:value:position)`, ';
            $sql .= 'p.`online_only`, p.`condition`, p.`customizable`, p.`uploadable_files`, p.`text_fields`, ';
            $sql .= 'psa.`out_of_stock`, p.`id_shop_default`, p.`advanced_stock_management`, psa.`depends_on_stock`, p.`advanced_stock_management`, MIN(wpl.`id_warehouse`) ';
            $sql .= 'FROM `' . _DB_PREFIX_ . 'product` p ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'product_lang` pl ON (p.`id_product` = pl.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'category_product` cp ON (p.`id_product` = cp.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl ON (cp.`id_category` = cl.`id_category`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'category` c ON (cp.`id_category` = c.`id_category`) ';
            // $sql .= 'RIGHT JOIN `' . _DB_PREFIX_ . 'category` c2 ON c2.`id_category` IN (' . $cat_ids . ') AND c.`nleft` >= c2.`nleft` ';
            // $sql .= 'AND c.`nright` <= c2.`nright`';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'specific_price` sp ON (p.`id_product` = sp.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'product_supplier` ps ON (p.`id_supplier` = ps.`id_supplier`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'supplier` su ON (p.`id_supplier` = su.`id_supplier`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'manufacturer` m ON (p.`id_manufacturer` = m.`id_manufacturer`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'stock_available` psa ON (p.`id_product` = psa.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'product_tag` pt ON (p.`id_product` = pt.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'tag` t ON (pt.`id_tag` = t.`id_tag`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'image` pi ON ( p.id_product = pi.id_product) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_product` fp ON p.`id_product` = fp.`id_product` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_lang` fl ON fp.`id_feature` = fl.`id_feature` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_value_lang` fvl ON fp.`id_feature_value` = fvl.`id_feature_value` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature` f ON fp.`id_feature` = f.`id_feature` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_value` fv ON fp.`id_feature_value` = fv.`id_feature_value` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'warehouse_product_location` wpl ON p.`id_product` = wpl.`id_product` ';
            $sql .= 'AND pl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'AND cl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'AND fl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'AND fvl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'AND p.`id_shop_default` = ' . $id_shop . ' ';
            $sql .= 'AND c.`id_shop_default` = ' . $id_shop . ' ';
            $sql .= 'WHERE c.`id_category` IN (' . $cat_ids . ') GROUP BY p.`id_product`';
            // $sql .= 'WHERE p.`id_product` > 0 GROUP BY p.`id_product`';
            // $sql .= 'WHERE p.`id_product` > ' . $last_id . ' GROUP BY p.`id_product` LIMIT 1000';

            $products = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS($sql);

            // foreach ($products as $key => $value) {
            //     $last_id = $products[$key]['id_product'];
            // }
            $csv->setCSV($products);
        }
        // } while (!empty($products));
        // var_dump(memory_get_usage());
        // var_dump(memory_get_peak_usage());
    }

    /**
     * Get Products
     */
    protected function csvMinprod()
    {
        require_once 'csvwrite.php';
        $csv_dir = csvexpert::$csv_dir . '/';
        $csv = new CSV($csv_dir, 'productexport.csv');
        $title = array('Product ID', 'Active (0/1)', 'Name *', 'Categories (x,y,z...)', 'Price tax excluded or Price tax included', 'Wholesale price', 'Reference #', 'Manufacturer', 'Unity', 'Short description', 'Description', 'Meta title', 'Meta keywords', 'Meta description', 'URL rewritten', 'Image URLs (x,y,z...)', 'Delete existing images (0=No, 1=Yes)', 'Feature(Name:Value:Position)', 'Out of stock', 'Quantity');
        $csv->setCSV(array(0 => $title));

        $cat_id = Configuration::get('TREE_CAT_CSVEXPERT');
        $cat_ids = $this->getCatSubcatId($cat_id);
        if ($cat_id) {
            $url = Tools::getShopDomainSsl(true);
            $id_lang = (int) $this->context->language->id;
            $id_shop = (int) $this->context->shop->id;
            $delete_existing_images = '1';
            $sql = 'SELECT p.`id_product`, p.`active`, pl.`name`, ';
            $sql .= 'GROUP_CONCAT(DISTINCT(cl.`name`) ORDER BY p.`id_category_default` ASC SEPARATOR ",") AS `category`, ';
            $sql .= 'p.`price`, p.`wholesale_price`, p.`reference`, ';
            $sql .= 'm.`name` AS `manufacturer`, ';
            $sql .= 'p.`unity`, ';
            $sql .= 'pl.`description_short`, pl.`description`, ';
            $sql .= 'pl.`meta_title`, pl.`meta_keywords`, pl.`meta_description`, pl.`link_rewrite`, ';
            $sql .= 'IF(pi.`id_product` IS NOT NULL, GROUP_CONCAT(DISTINCT(CONCAT( "' . $url . '/img/p/",MID(pi.id_image,1,1),"/", IF (LENGTH(pi.id_image)>1, CONCAT(MID(pi.id_image,2,1),"/"),""),IF (LENGTH(pi.id_image)>2,CONCAT(MID(pi.id_image,3,1),"/"),""),IF (LENGTH(pi.id_image)>3,CONCAT(MID(pi.id_image,4,1),"/"),""),IF (LENGTH(pi.id_image)>4,CONCAT(MID(pi.id_image,5,1),"/"),""), pi.id_image, ".jpg" )) ORDER BY pi.`cover` DESC, pi.`position` ASC SEPARATOR ","), "") AS url_image, ';
            $sql .= 'IF(pi.`id_product` IS NOT NULL,' . $delete_existing_images . ', "") AS `delete existing img (0=no, 1=yes)`, ';
            // $sql .= 'GROUP_CONCAT(DISTINCT(CONCAT((fl.`name`), ":", (REPLACE(fvl.`value`,","," &")))) SEPARATOR ",") AS `feature(name:value)`, ';
            $sql .= 'GROUP_CONCAT(DISTINCT(CONCAT((fl.`name`), ":", (REPLACE(fvl.`value`,","," &")), ":" , (f.position))) SEPARATOR ",") AS `feature(name:value:position)`, ';
            $sql .= 'psa.`out_of_stock`, psa.`quantity` ';
            $sql .= 'FROM `' . _DB_PREFIX_ . 'product` p ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'product_lang` pl ON (p.`id_product` = pl.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'category_product` cp ON (p.`id_product` = cp.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl ON (cp.`id_category` = cl.`id_category`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'category` c ON (cp.`id_category` = c.`id_category`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'manufacturer` m ON (p.`id_manufacturer` = m.`id_manufacturer`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'stock_available` psa ON (p.`id_product` = psa.`id_product`) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'image` pi ON ( p.id_product = pi.id_product) ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_product` fp ON p.`id_product` = fp.`id_product` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_lang` fl ON fp.`id_feature` = fl.`id_feature` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_value_lang` fvl ON fp.`id_feature_value` = fvl.`id_feature_value` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature` f ON fp.`id_feature` = f.`id_feature` ';
            $sql .= 'LEFT JOIN `' . _DB_PREFIX_ . 'feature_value` fv ON fp.`id_feature_value` = fv.`id_feature_value` ';
            $sql .= 'AND pl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'AND cl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'AND fl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'AND fvl.`id_lang` = ' . $id_lang . ' ';
            $sql .= 'WHERE c.`id_category` IN (' . $cat_ids . ') GROUP BY p.`id_product` ';
            $sql .= 'ORDER BY c.`id_parent`, `manufacturer`, `category`, pl.`name`';

            $products = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS($sql);

            foreach ($products as $key => $value) {
                if ($products[$key]['name']) {
                    $products[$key]['name'] = trim(mb_ereg_replace('\\s+', ' ', $value['name']));
                }
                if ($products[$key]['category']) {
                    $products[$key]['category'] = $this->cleanCat($value['category']);
                }
                if ($products[$key]['reference']) {
                    $products[$key]['reference'] = $this->cleanShortDesc($value['reference']);
                }
                if ($products[$key]['description_short']) {
                    $products[$key]['description_short'] = $this->cleanShortDesc($value['description_short']);
                }
                if ($products[$key]['description']) {
                    $products[$key]['description'] = $this->cleanDesc($value['description']);
                }
                if ($products[$key]['title']) {
                    $products[$key]['title'] = $this->cleanShortDesc($value['title']);
                }
                if ($products[$key]['meta_keywords']) {
                    $products[$key]['meta_keywords'] = $this->cleanKeywords($value["meta_keywords"]);
                }
                if ($products[$key]['meta_description']) {
                    $products[$key]['meta_description'] = $this->cleanShortDesc($value['meta_description']);
                }
            }

            $csv->setCSV($products);
        }
    }

    /**
     * Get Attributes
     */
    protected function csvAttributes()
    {
        $cat_id = Configuration::get('TREE_CAT_CSVEXPERT');
        $cat_ids = $this->getCatSubcatId($cat_id);
        require_once 'csvwrite.php';
        $csv_dir = csvexpert::$csv_dir . '/';
        $csv = new CSV($csv_dir, 'attributes.csv');
        $title = array('Product ID*', 'Attribute (Name:Type:Position)*', 'Value (Value:Position)*', 'Supplier reference', 'Reference', 'EAN13', 'UPC', 'Wholesale price', 'Impact on price', 'Ecotax', 'Quantity', 'Minimal quantity', 'Impact on weight', 'Default (0=No, 1=Yes)', 'Combination available date', 'Image position', 'Image URL', 'Delete existing images (0=No, 1=Yes)', 'ID/Name of shop', 'Advanced Stock Managment', 'Depends on stock', 'Warehouse');
        $csv->setCSV(array(0 => $title));
        if ($cat_ids) {
            $url = Tools::getShopDomainSsl(true);
            $delete_existing_images = '1';
            $id_lang = (int) $this->context->language->id;
            $id_shop = (int) $this->context->shop->id;

            $sql = 'SELECT pa.`id_product`,
            GROUP_CONCAT(DISTINCT(CONCAT((agl.`name`), ":", (ag.`group_type`), ":" , (ag.`position`))) SEPARATOR ",") AS `Attribute (Name:Type:Position)*`,
            GROUP_CONCAT(DISTINCT(CONCAT((al.`name`), ":" , (ag.`position`))) SEPARATOR ",") AS `Value (Value:Position)*`,
            ps.`product_supplier_reference`, pa.`reference`, pa.`ean13`, pa.`upc`, pa.`wholesale_price`, pa.`price`, pa.`ecotax`, pa.`quantity`, pa.`minimal_quantity`, pa.`weight`, pa.`default_on`, pa.`available_date`, pi.`position`,
            IF(pi.`id_product` IS NOT NULL, GROUP_CONCAT(DISTINCT(CONCAT( "' . $url . '/img/p/",MID(pi.id_image,1,1),"/", IF (LENGTH(pi.id_image)>1, CONCAT(MID(pi.id_image,2,1),"/"),""),IF (LENGTH(pi.id_image)>2,CONCAT(MID(pi.id_image,3,1),"/"),""),IF (LENGTH(pi.id_image)>3,CONCAT(MID(pi.id_image,4,1),"/"),""),IF (LENGTH(pi.id_image)>4,CONCAT(MID(pi.id_image,5,1),"/"),""), pi.id_image, ".jpg" )) SEPARATOR ","), "") AS url_image,
            IF(pi.`id_product` IS NOT NULL,' . $delete_existing_images . ', "") AS `delete existing img (0=no, 1=yes)`
            FROM `' . _DB_PREFIX_ . 'product_attribute` pa
            LEFT JOIN `' . _DB_PREFIX_ . 'product_attribute_combination` pac ON pac.`id_product_attribute` = pa.`id_product_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute` a ON a.`id_attribute` = pac.`id_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_lang` al ON a.`id_attribute` = al.`id_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group` ag ON ag.`id_attribute_group` = a.`id_attribute_group`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group_lang` agl ON ag.`id_attribute_group` = agl.`id_attribute_group`
            LEFT JOIN `' . _DB_PREFIX_ . 'product_supplier` ps ON ps.`id_product_attribute` = pa.`id_product_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'product_attribute_image` agi ON pa.`id_product_attribute` = agi.`id_product_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'image` pi ON agi.`id_image` = pi.`id_image`
            LEFT JOIN `' . _DB_PREFIX_ . 'category_product` cp ON cp.`id_product` = pa.`id_product`
            WHERE cp.`id_category` IN (' . $cat_ids . ')
            AND  al.`id_lang` = ' . $id_lang . '
            AND agl.`id_lang` = ' . $id_lang . '
            GROUP BY pa.`id_product_attribute`ORDER BY pa.`id_product` ASC';
            $attributes = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS($sql);

            $csv->setCSV($attributes);
        }
    }

    /**
     * Get Manufacturers
     */
    protected function csvManufacturers()
    {
        require_once 'csvwrite.php';
        $csv_dir = csvexpert::$csv_dir . '/';
        $csv = new CSV($csv_dir, 'manufacturers.csv');
        $title = array('Manufacturer ID', 'Active (0/1)', 'Name *', 'Description', 'Short description', 'Meta title', 'Meta keywords', 'Meta description', 'Image URL');
        $csv->setCSV(array(0 => $title));

        $url = Tools::getShopDomainSsl(true);
        $id_lang = (int) $this->context->language->id;

        $sql = 'SELECT
            m.`id_manufacturer`,
            m.`active`,
            m.`name`,
            ml.`description`,
            ml.`short_description`,
            ml.`meta_title`,
            ml.`meta_keywords`,
            ml.`meta_description`
            FROM
            `' . _DB_PREFIX_ . 'manufacturer` m
            LEFT JOIN `' . _DB_PREFIX_ . 'manufacturer_lang` ml ON
            ml.`id_manufacturer` = m.`id_manufacturer`
            WHERE
            ml.`id_lang` = ' . $id_lang . '
            ORDER BY
            m.`id_manufacturer` ASC';

        $manufacturers = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS($sql);
        foreach ($manufacturers as $key => $value) {
            if (file_exists(_PS_MANU_IMG_DIR_ . (int) $manufacturers[$key]['id_manufacturer'] . '.jpg')) {
                $manufacturers[$key]['image'] = $url . '/img/m/' . $manufacturers[$key]['id_manufacturer'] . '.jpg';
            } else {
                $manufacturers[$key]['image'] = '';
            }
        }

        $csv->setCSV($manufacturers);

    }

    /**
     * Delete all csv
     */
    protected function csvDelete()
    {
        $csvs = glob($_SERVER['DOCUMENT_ROOT'] . "/modules/csvexpert/csv/*");
        foreach ($csvs as $csv) {
            @unlink($csv);
        }
    }
}
